@@ -21,14 +21,10 @@ module Numeric.Algebra.Class
   , Coalgebra(..)
   ) where
 
-import Control.Applicative
 import Data.Foldable hiding (sum, concat)
-import Data.Functor.Representable
-import Data.Functor.Representable.Trie
 import Data.Int
 import Data.IntMap (IntMap)
 import Data.IntSet (IntSet)
-import Data.Key
 import Data.Map (Map)
 import Data.Monoid (mappend)
 -- import Data.Semigroup.Foldable
@@ -37,7 +33,7 @@ import Data.Semigroup.Foldable
 import Data.Set (Set)
 import Data.Word
 import Numeric.Additive.Class
-import Numeric.Natural.Internal
+import Numeric.Natural
 import Prelude hiding ((*), (+), negate, subtract,(-), recip, (/), foldr, sum, product, replicate, concat)
 import qualified Data.IntMap as IntMap
 import qualified Data.IntSet as IntSet
@@ -55,7 +51,7 @@ class Multiplicative r where
 
 -- class Multiplicative r => PowerAssociative r where
   -- pow1p x n = pow x (1 + n)
-  pow1p :: Whole n => r -> n -> r
+  pow1p :: r -> Natural -> r
   pow1p x0 y0 = f x0 (y0 Prelude.+ 1) where
     f x y 
       | even y = f (x * x) (y `quot` 2)
@@ -149,8 +145,6 @@ instance (Multiplicative a, Multiplicative b, Multiplicative c, Multiplicative d
 
 instance Algebra r a => Multiplicative (a -> r) where
   f * g = mult $ \a b -> f a * g b
-instance (HasTrie a, Algebra r a) => Multiplicative (a :->: r) where
-  f * g = tabulate $ mult $ \a b -> index f a * index g b
 
 -- | A pair of an additive abelian semigroup, and a multiplicative semigroup, with the distributive laws:
 -- 
@@ -186,7 +180,6 @@ instance (Semiring a, Semiring b, Semiring c) => Semiring (a, b, c)
 instance (Semiring a, Semiring b, Semiring c, Semiring d) => Semiring (a, b, c, d)
 instance (Semiring a, Semiring b, Semiring c, Semiring d, Semiring e) => Semiring (a, b, c, d, e)
 instance Algebra r a => Semiring (a -> r) 
-instance (HasTrie a, Algebra r a) => Semiring (a :->: r) 
 
 -- | An associative algebra built with a free module over a semiring
 class Semiring r => Algebra r a where
@@ -254,9 +247,6 @@ class Semiring r => Coalgebra r c where
 instance Algebra r m => Coalgebra r (m -> r) where
   comult k f g = k (f * g)
 
-instance (HasTrie m, Algebra r m) => Coalgebra r (m :->: r) where
-  comult k f g = k (f * g)
-
 -- instance Coalgebra () c where comult _ _ _ = ()
 -- instance (Algebra r b, Coalgebra r c) => Coalgebra (b -> r) c where comult f c1 c2 b = comult (`f` b) c1 c2 
 
@@ -310,7 +300,7 @@ instance LeftModule Natural Natural where
   (.*) = (*)
 
 instance LeftModule Natural Integer where 
-  Natural n .* m = n * m
+  n .* m = toInteger n * m
 
 instance LeftModule Integer Integer where 
   (.*) = (*) 
@@ -381,9 +371,6 @@ instance Semiring r => LeftModule r () where
 instance LeftModule r m => LeftModule r (e -> m) where 
   (.*) m f e = m .* f e
 
-instance (HasTrie e, LeftModule r m) => LeftModule r (e :->: m) where 
-  (.*) m f = tabulate $ \e -> m .* index f e
-
 instance Additive m => LeftModule () m where 
   _ .* a = a
 
@@ -410,7 +397,7 @@ instance RightModule Natural Bool where
 
 instance RightModule Natural Natural where (*.) = (*)
 
-instance RightModule Natural Integer where n *. Natural m = n * m
+instance RightModule Natural Integer where n *. m = n * fromIntegral m
 
 instance RightModule Integer Integer where (*.) = (*) 
 
@@ -460,9 +447,6 @@ instance Semiring r => RightModule r () where
 instance RightModule r m => RightModule r (e -> m) where 
   (*.) f m e = f e *. m
 
-instance (HasTrie e, RightModule r m) => RightModule r (e :->: m) where 
-  (*.) f m = tabulate $ \e -> index f e *. m
-
 instance Additive m => RightModule () m where 
   (*.) = const
 
@@ -491,18 +475,18 @@ instance (LeftModule r m, RightModule r m) => Module r m
 class (LeftModule Natural m, RightModule Natural m) => Monoidal m where
   zero :: m
 
-  sinnum :: Whole n => n -> m -> m
+  sinnum :: Natural -> m -> m
   sinnum 0 _  = zero
   sinnum n x0 = f x0 n
     where
       f x y
         | even y = f (x + x) (y `quot` 2)
         | y == 1 = x
-        | otherwise = g (x + x) (unsafePred y `quot` 2) x
+        | otherwise = g (x + x) (pred y `quot` 2) x
       g x y z
         | even y = g (x + x) (y `quot` 2) z
         | y == 1 = x + z
-        | otherwise = g (x + x) (unsafePred y `quot` 2) (x + z)
+        | otherwise = g (x + x) (pred y `quot` 2) (x + z)
 
   sumWith :: Foldable f => (a -> m) -> f a -> m
   sumWith f = foldl' (\b a -> b + f a) zero
@@ -521,7 +505,7 @@ instance Monoidal Bool where
 
 instance Monoidal Natural where
   zero = 0
-  sinnum n r = toNatural n * r
+  sinnum n r = fromIntegral n * r
 
 instance Monoidal Integer where 
   zero = 0
@@ -572,11 +556,6 @@ instance Monoidal r => Monoidal (e -> r) where
   sumWith f xs e = sumWith (`f` e) xs
   sinnum n r e = sinnum n (r e)
 
-instance (HasTrie e, Monoidal r) => Monoidal (e :->: r) where
-  zero = pure zero
-  sumWith f xs = tabulate $ \e -> sumWith (\a -> index (f a) e) xs
-  sinnum n r = tabulate $ sinnum n . index r
-
 instance Monoidal () where 
   zero = ()
   sinnum _ () = ()
